"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var axis_utils_1 = require("../../axes/axis_utils");
var commons_1 = require("../../utils/commons");
var domain_1 = require("../../utils/domain");
/**
 * Merge X domain value between a set of chart specification.
 */
function mergeXDomain(specs, xValues, xDomain) {
    var mainXScaleType = convertXScaleTypes(specs);
    if (!mainXScaleType) {
        throw new Error('Cannot merge the domain. Missing X scale types');
    }
    var values = __spread(xValues.values());
    var seriesXComputedDomains;
    var minInterval = 0;
    if (mainXScaleType.scaleType === "ordinal" /* Ordinal */) {
        seriesXComputedDomains = domain_1.computeOrdinalDataDomain(values, commons_1.identity, false, true);
        if (xDomain) {
            if (Array.isArray(xDomain)) {
                seriesXComputedDomains = xDomain;
            }
            else {
                throw new Error('xDomain for ordinal scale should be an array of values, not a DomainRange object');
            }
        }
    }
    else {
        seriesXComputedDomains = domain_1.computeContinuousDataDomain(values, commons_1.identity, true);
        if (xDomain) {
            if (!Array.isArray(xDomain)) {
                var _a = __read(seriesXComputedDomains, 2), computedDomainMin = _a[0], computedDomainMax = _a[1];
                if (axis_utils_1.isCompleteBound(xDomain)) {
                    if (xDomain.min > xDomain.max) {
                        throw new Error('custom xDomain is invalid, min is greater than max');
                    }
                    seriesXComputedDomains = [xDomain.min, xDomain.max];
                }
                else if (axis_utils_1.isLowerBound(xDomain)) {
                    if (xDomain.min > computedDomainMax) {
                        throw new Error('custom xDomain is invalid, custom min is greater than computed max');
                    }
                    seriesXComputedDomains = [xDomain.min, computedDomainMax];
                }
                else if (axis_utils_1.isUpperBound(xDomain)) {
                    if (computedDomainMin > xDomain.max) {
                        throw new Error('custom xDomain is invalid, computed min is greater than custom max');
                    }
                    seriesXComputedDomains = [computedDomainMin, xDomain.max];
                }
            }
            else {
                throw new Error('xDomain for continuous scale should be a DomainRange object, not an array');
            }
        }
        minInterval = findMinInterval(values);
    }
    return {
        type: 'xDomain',
        scaleType: mainXScaleType.scaleType,
        isBandScale: mainXScaleType.isBandScale,
        domain: seriesXComputedDomains,
        minInterval: minInterval,
        timeZone: mainXScaleType.timeZone,
    };
}
exports.mergeXDomain = mergeXDomain;
/**
 * Find the minimum interval between xValues.
 * Default to 0 if an empty array, 1 if one item array
 */
function findMinInterval(xValues) {
    var valuesLength = xValues.length;
    if (valuesLength <= 0) {
        return 0;
    }
    if (valuesLength === 1) {
        return 1;
    }
    var sortedValues = xValues.slice().sort(commons_1.compareByValueAsc);
    var i;
    var minInterval = Math.abs(sortedValues[1] - sortedValues[0]);
    for (i = 1; i < valuesLength - 1; i++) {
        var current = sortedValues[i];
        var next = sortedValues[i + 1];
        var interval = Math.abs(next - current);
        minInterval = Math.min(minInterval, interval);
    }
    return minInterval;
}
exports.findMinInterval = findMinInterval;
/**
 * Convert the scale types of a set of specification to a generic one.
 * If there is at least one bar series type, than the response will specity
 * that the coerched scale is a `scaleBand` (each point needs to have a surrounding empty
 * space to draw the bar width).
 * If there are multiple continuous scale types, is coerched to linear.
 * If there are at least one Ordinal scale type, is coerched to ordinal.
 * If none of the above, than coerch to the specified scale.
 * @returns {ChartScaleType}
 */
function convertXScaleTypes(specs) {
    var seriesTypes = new Set();
    var scaleTypes = new Set();
    var timeZones = new Set();
    specs.forEach(function (spec) {
        seriesTypes.add(spec.seriesType);
        scaleTypes.add(spec.xScaleType);
        if (spec.timeZone) {
            timeZones.add(spec.timeZone.toLowerCase());
        }
    });
    if (specs.length === 0 || seriesTypes.size === 0 || scaleTypes.size === 0) {
        return null;
    }
    var isBandScale = seriesTypes.has('bar');
    if (scaleTypes.size === 1) {
        var scaleType = scaleTypes.values().next().value;
        var timeZone = void 0;
        if (scaleType === "time" /* Time */) {
            if (timeZones.size > 1) {
                timeZone = 'utc';
            }
            else {
                timeZone = timeZones.values().next().value;
            }
        }
        return { scaleType: scaleType, isBandScale: isBandScale, timeZone: timeZone };
    }
    if (scaleTypes.size > 1 && scaleTypes.has("ordinal" /* Ordinal */)) {
        return { scaleType: "ordinal" /* Ordinal */, isBandScale: isBandScale };
    }
    return { scaleType: "linear" /* Linear */, isBandScale: isBandScale };
}
exports.convertXScaleTypes = convertXScaleTypes;
//# sourceMappingURL=x_domain.js.map